#######################################################################
# Copyright (c) 2024 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

include IncidentsHelper

module Incidents
  class IncidentDruidRetriever
    def self.get_data(incident, options = {}, user, get_assets_ip)
      elements = get_incident_druid_data(incident, options, user, get_assets_ip)
      elements
    end
  
    def self.get_incident_druid_data(incident, search_params, current_user, get_assets_ip)
      data = []
    
      # Mapping of fields that can change depending on the module or data source
      field_map = {
        priority: [:priority, :syslogseverity_text], # Priority can be one of these fields
        msg: [:msg, :message],                       # Msg can be one of these fields
      }
  
      @objects = RedborderObject.where("user_id = ? OR user_id IS NULL", current_user.id)
  
      incident_modules = Redborder::Module.registered_modules.select { |_, v| v[:columns] && v[:columns][:incident_uuid] }
  
      incident_modules.each do |mod| 
        source = Redborder::DruidSource.get(source: mod.first)
        filter = Redborder::Filter.new(source, {})
        filter.add(:incident_uuid, incident.uuid)
  
        start_time = incident.created_at - 1.day
        end_time = incident.created_at + 1.day
  
        # group_by: @source.default_columns_raw,
        data += source.where({
          query_type: :group_by,
          range: :custom,
          start_time: start_time,
          end_time: end_time,
          granularity: :minute,
          aggregation: source.default_aggregation,
          group_by: source.default_columns_incidents,
          search: filter.get,
          user: current_user
        }).map do |t|
          url_filters = {}
          url_filters[:start_time] = start_time
          url_filters[:end_time] = end_time
  
          sig_id = 'N/A'
          if t.methods.include?(:sig_id)
            sig_id = t.send(:sig_id) || 'N/A'
          end
  
          # We use the map_dynamic_fields method to map the fields correctly
          priority = map_dynamic_fields(t, priority: field_map[:priority])
          msg = map_dynamic_fields(t, msg: field_map[:msg])
  
          source_name = mod.last[:incident_source_name] rescue "N/A"
          
          module_name = mod.first
  
          timestamp = t.send(:timestamp) 
  
          # Observables are define in the columns of each module
          observables = []
          assets = []
          mod.last[:columns].select { |column, cfg| cfg[:observable_type] }.each do |column, cfg|
            asset = nil
            url_filters[column] = t.send(column)
  
            # We check if this is an asset
            if cfg[:object_type] && t.methods.include?(column) && t.send(column)
              asset = RedborderObject.where("user_id = ? OR user_id IS NULL", current_user.id).where("value = ? OR ip_addresses LIKE ?", t.send(column), "%#{t.send(column)}%").first
            end          
            if asset
              asset_type = asset.object_type || cfg[:object_type]
              if get_assets_ip
                assets.push({ asset.name => { asset_type: asset_type, asset_value: t.send(column) } }) if t.methods.include?(column) && t.send(column)
              else
                assets.push({ asset.name => asset_type }) if t.methods.include?(column) && t.send(column)
              end
            else
              observables.push( { t.send(column) => cfg[:observable_type] }) if t.methods.include?(column) && t.send(column)
            end
          end
  
          url_filters["incident_uuid"] = incident.uuid
  
          { timestamp: timestamp, priority: priority, msg: msg, source: source_name, assets: assets, observables: observables, module_name: module_name, url_filters: url_filters, sig_id: sig_id }
        end
      end
      
      # Filter by search
      if search_params && search_params[:source]
        data = data.select { |element| element[:source] == search_params[:source] }
      end
  
      if search_params && search_params[:priority]
        data = data.select { |element| element[:priority] == search_params[:priority] }
      end
  
      data
    end
  end
end