#######################################################################
# Copyright (c) 2024 ENEO Tecnologia S.L.
# This file is part of redBorder.
# redBorder is free software: you can redistribute it and/or modify
# it under the terms of the GNU Affero General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# redBorder is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU Affero General Public License for more details.
# You should have received a copy of the GNU Affero General Public License
# along with redBorder. If not, see <http://www.gnu.org/licenses/>.
#######################################################################

include IncidentsHelper

module Incidents
  class IncidentOutlierRetriever
    def self.get_data(incident, user)
      observables, assets = fetch_outliers_assets_and_observables(incident, user)
      compile_outliers_data(incident, assets, observables)
    end
  
    private
  
    def self.fetch_outliers_assets_and_observables(incident, user)
      observables = []
      assets = []
  
      incident.outliers.each do |outlier|
        if outlier.observables.is_a?(Array)
          outlier.observables.each do |observable_array|
            observable_array.reject! do |observable|
              observable_key = observable.keys.first
              asset = RedborderObject.find_by(value: observable_key, user: user)
              assets << { observable_key => "NetObject" } if asset 
            end
          end
          observables.concat(outlier.observables)
        end
      end
  
      [observables.flatten.uniq, assets.uniq]
    end
  
    def self.compile_outliers_data(incident, assets, observables)
      elements = []
  
      incident.outliers.each do |outlier|
        sensor_name = outlier.sensor_name
        sensor_uuid = outlier.sensor_uuid
        elements << {
          timestamp: outlier.timestamp,
          priority: outlier.priority || "high",
          msg: "#{I18n.t('controllers.outlier_on')} #{sensor_name}",
          source: Redborder::Module.registered_modules[:flow][:incident_source_name],
          assets: assets,
          observables: observables,
          module_name: :flow,
          url_filter: get_incident_filters_outliers(
            incident, sensor_name, sensor_uuid, outlier.lan_ip_regex, outlier.wan_ip_regex
          )
        }
      end
  
      elements
    end
  end
end